#!/usr/bin/env python
# -*- coding: utf-8 -*-

from builtins import str
import time

from db_paribhasha import credentials
from db_paribhasha.dynamo import dynamo_helpers as db_helpers
from db_paribhasha.dynamo.export_workflow import ExportWorkflow
from db_paribhasha.error_handler import OperationNotPerformedError


def create_table():
    """
    Creates table if table doesn't exist
    """

    if ExportWorkflow.exists():
        raise OperationNotPerformedError("Table already exists")
    ExportWorkflow.create_table(
        read_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP,
        write_capacity_units=credentials.DYNAMO_GENERIC_RW_CAP)


def get_export_workflow_detail(export_workflow_id):
    """
    Returns the detail of a given workflow id.

    Parameters
    ----------
    export_workflow_id: String
        Unique export id

    Returns
    -------
    export_workflow: Pynamo object
        export workflow for the given unique ID.
    """
    org_id, user_id, created_timestamp = export_workflow_id.split('-')
    export_workflow = ExportWorkflow.get(
        int(org_id), user_id + '-' + created_timestamp)
    return export_workflow


def create_export_workflow(org_id,
                           email_id,
                           workflow_adapter_id,
                           run_id,
                           state='init'):
    """
    Creates an export workflow for a given user for an adapter id

    Parameters
    ----------
    org_id: Integer
        Organization ID
    email_id: String
        Email ID of the creating user
    workflow_adapter_id: String
        The adapter ID to create workflow for
    run_id: Integer
        The Run ID associated with export
    state: String
        State for the workflow, optional: default: init

    Returns
    -------
    export_workflow_id: String
        Created export workflow ID
    """
    added_timestamp = int(round(time.time() * 1000))
    last_modified_timestamp = added_timestamp
    calculated_id = str(email_id) + '-' + str(added_timestamp)
    export_workflow =\
        ExportWorkflow(org_id=int(org_id), created_ts=added_timestamp,
                       calculated_id=calculated_id,
                       modified_ts=last_modified_timestamp,
                       user_email=email_id,
                       export_adapter_id=workflow_adapter_id,
                       state=state,
                       run_id=run_id)
    db_helpers.save_dynamo_object(export_workflow)
    return '{}-{}'.format(str(org_id), calculated_id)


def update_export_workflow(export_wf_id, new_state):
    """
    Updates a given export workflow

    Parameters
    ----------
    export_wf_id : [String]
        Expport workflow id to update
    new_state : String
        State to update

    """

    export_workflow = get_export_workflow_detail(export_wf_id)
    export_workflow.state = new_state
    db_helpers.save_dynamo_object(export_workflow)
